@@ -31,9 +31,13 @@
 #include "gstalsaclock.h"
 #include "gstalsamixer.h"
 
+/* all this ifdef'ed stuff causes segfaults because of alsa bug 389, see
+ * https://bugtrack.alsa-project.org/alsa-bug/bug_view_page.php?bug_id=0000389
+ */
+#ifdef ALSA_BUG_389_FIXED
 #define ALSA_DEBUG_FLUSH(this) G_STMT_START{ \
   gchar *__str; \
-  ssize_t __size; \
+  size_t __size; \
   __size = snd_output_buffer_string (this->out, &__str); \
   if (__size > 0) { \
     GST_DEBUG_OBJECT (this, "%*s", __size, __str); \
@@ -41,6 +45,7 @@
       GST_ERROR_OBJECT (this, "error flushing output buffer"); \
   } \
 }G_STMT_END
+#endif
 
 /* GObject functions */
 static void gst_alsa_class_init (gpointer g_class, gpointer class_data);
@@ -1085,7 +1090,8 @@ gst_alsa_change_state (GstElement * element)
       if (!(GST_FLAG_IS_SET (element, GST_ALSA_RUNNING) ||
               gst_alsa_start_audio (this)))
         return GST_STATE_FAILURE;
-      this->transmitted = 0;
+      this->played = 0;
+      this->captured = 0;
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
       if (snd_pcm_state (this->handle) == SND_PCM_STATE_PAUSED) {
@@ -1130,7 +1136,7 @@ gst_alsa_change_state (GstElement * element)
       break;
 
     default:
-      g_assert_not_reached ();
+      break;
   }
 
   if (GST_ELEMENT_CLASS (parent_class)->change_state)
@@ -1212,6 +1218,7 @@ gst_alsa_start (GstAlsa * this)
       ERROR_CHECK (snd_pcm_prepare (this->handle), "error preparing: %s");
     case SND_PCM_STATE_SUSPENDED:
     case SND_PCM_STATE_PREPARED:
+      this->captured = 0;
       ERROR_CHECK (snd_pcm_start (this->handle), "error starting playback: %s");
       break;
     case SND_PCM_STATE_PAUSED:
@@ -1249,8 +1256,6 @@ gst_alsa_xrun_recovery (GstAlsa * this)
     gettimeofday (&now, 0);
     snd_pcm_status_get_trigger_tstamp (status, &tstamp);
     timersub (&now, &tstamp, &diff);
-    GST_INFO_OBJECT (this, "alsa: xrun of at least %.3f msecs",
-        diff.tv_sec * 1000 + diff.tv_usec / 1000.0);
 
     /* if we're allowed to recover, ... */
     if (this->autorecover) {
@@ -1264,12 +1269,11 @@ gst_alsa_xrun_recovery (GstAlsa * this)
         this->period_count *= 2;
       }
     }
-  }
-
-  if (!(gst_alsa_stop_audio (this) && gst_alsa_start_audio (this))) {
-    GST_ELEMENT_ERROR (this, RESOURCE, FAILED, (NULL),
-        ("Error restarting audio after xrun"));
-    return FALSE;
+    if (!(gst_alsa_stop_audio (this) && gst_alsa_start_audio (this))) {
+      GST_ELEMENT_ERROR (this, RESOURCE, FAILED, (NULL),
+          ("Error restarting audio after xrun"));
+      return FALSE;
+    }
   }
 
   return TRUE;
@@ -1300,8 +1304,10 @@ gst_alsa_open_audio (GstAlsa * this)
 
   GST_INFO ("Opening alsa device \"%s\"...", this->device);
 
+#ifdef ALSA_BUG_389_FIXED
   ERROR_CHECK (snd_output_buffer_open (&this->out),
       "error opening log output: %s");
+#endif
 
   if ((ret = snd_pcm_open (&this->handle, this->device,
               GST_ALSA_GET_CLASS (this)->stream, SND_PCM_NONBLOCK)) < 0) {
@@ -1346,15 +1352,19 @@ gst_alsa_open_audio (GstAlsa * this)
 void
 gst_alsa_sw_params_dump (GstAlsa * this, snd_pcm_sw_params_t * sw_params)
 {
+#ifdef ALSA_BUG_389_FIXED
   snd_pcm_sw_params_dump (sw_params, this->out);
   ALSA_DEBUG_FLUSH (this);
+#endif
 }
 
 void
 gst_alsa_hw_params_dump (GstAlsa * this, snd_pcm_hw_params_t * hw_params)
 {
+#ifdef ALSA_BUG_389_FIXED
   snd_pcm_hw_params_dump (hw_params, this->out);
   ALSA_DEBUG_FLUSH (this);
+#endif
 }
 
 /* if someone finds an easy way to merge this with _set_hw_params, go ahead */
@@ -1580,6 +1590,7 @@ gst_alsa_drain_audio (GstAlsa * this)
       break;
   }
 
+  GST_DEBUG ("stopped alsa");
   GST_FLAG_UNSET (this, GST_ALSA_RUNNING);
   return TRUE;
 }
@@ -1611,7 +1622,9 @@ gst_alsa_stop_audio (GstAlsa * this)
 static gboolean
 gst_alsa_close_audio (GstAlsa * this)
 {
+#ifdef ALSA_BUG_389_FIXED
   gint err;
+#endif
 
   /* if there's no pads, we never open. So we don't close either. */
   if (!gst_element_get_pad_list (GST_ELEMENT (this)))
@@ -1620,11 +1633,13 @@ gst_alsa_close_audio (GstAlsa * this)
   g_return_val_if_fail (this != NULL, FALSE);
   g_return_val_if_fail (this->handle != NULL, FALSE);
 
+#ifdef ALSA_BUG_389_FIXED
   ALSA_DEBUG_FLUSH (this);
   err = snd_output_close (this->out);
   if (err != 0)
     GST_ERROR_OBJECT (this, "failed to close debugging output: %s",
         snd_strerror (err));
+#endif
   ERROR_CHECK (snd_pcm_close (this->handle), "Error closing device: %s");
 
   this->handle = NULL;
