@@ -42,7 +42,7 @@ enum
   ARG_0,
   ARG_CONSECUTIVE,
   ARG_SKIP,
-  ARG_DROP_INCOMPLETE,
+  ARG_DROP_INCOMPLETE
 };
 
 static GstElementDetails gst_dv1394src_details =
@@ -63,7 +63,7 @@ gst_dv1394src_factory (void)
         GST_PAD_SRC,
         GST_PAD_ALWAYS,
         GST_STATIC_CAPS ("dv1394src",
-            "video/dv",
+            "video/x-dv",
             gst_props_new ("format", GST_PROPS_LIST (G_TYPE_STRING ("NTSC"),
                     G_TYPE_STRING ("PAL")
                 ), NULL)
@@ -107,6 +107,7 @@ gst_dv1394src_get_type (void)
       0,
       (GInstanceInitFunc) gst_dv1394src_init,
     };
+
     gst_dv1394src_type =
         g_type_register_static (GST_TYPE_ELEMENT, "DV1394Src",
         &gst_dv1394src_info, 0);
@@ -155,6 +156,7 @@ gst_dv1394src_init (GstDV1394Src * dv1394src)
 {
   dv1394src->srcpad = gst_pad_new ("src", GST_PAD_SRC);
   gst_pad_set_get_function (dv1394src->srcpad, gst_dv1394src_get);
+  gst_pad_use_explicit_caps (dv1394src->srcpad);
   gst_element_add_pad (GST_ELEMENT (dv1394src), dv1394src->srcpad);
 
   dv1394src->card = 0;
@@ -251,9 +253,9 @@ gst_dv1394src_iso_receive (raw1394handle_t handle, int channel, size_t len,
           // PAL
           dv1394src->frameSize = PAL_FRAMESIZE;
           GST_DEBUG ("PAL data");
-          if (gst_pad_try_set_caps (dv1394src->srcpad,
-                  gst_caps_new_simple ("video/dv",
-                      "format", G_TYPE_STRING, "PAL", NULL)) <= 0) {
+          if (!gst_pad_set_explicit_caps (dv1394src->srcpad,
+                  gst_caps_new_simple ("video/x-dv",
+                      "format", G_TYPE_STRING, "PAL", NULL))) {
             GST_ELEMENT_ERROR (dv1394src, CORE, NEGOTIATION, (NULL),
                 ("Could not set source caps for PAL"));
             return 0;
@@ -263,9 +265,9 @@ gst_dv1394src_iso_receive (raw1394handle_t handle, int channel, size_t len,
           dv1394src->frameSize = NTSC_FRAMESIZE;
           GST_DEBUG
               ("NTSC data [untested] - please report success/failure to <dan@f3c.com>");
-          if (gst_pad_try_set_caps (dv1394src->srcpad,
-                  gst_caps_new_simple ("video/dv", "format", G_TYPE_STRING,
-                      "NTSC", NULL)) <= 0) {
+          if (!gst_pad_set_explicit_caps (dv1394src->srcpad,
+                  gst_caps_new_simple ("video/x-dv", "format", G_TYPE_STRING,
+                      "NTSC", NULL))) {
             GST_ELEMENT_ERROR (dv1394src, CORE, NEGOTIATION, (NULL),
                 ("Could not set source caps for NTSC"));
             return 0;
