@@ -28,7 +28,7 @@
 /* elementfactory information */
 GstElementDetails gst_speexenc_details = {
   "speex audio encoder",
-  "Codec/Audio/Encoder",
+  "Codec/Encoder/Audio",
   ".speex",
   "Wim Taymans <wim.taymans@chello.be>",
 };
@@ -43,8 +43,8 @@ enum
 
 enum
 {
-  ARG_0,
-  /* FILL ME */
+  ARG_0
+      /* FILL ME */
 };
 
 static void gst_speexenc_base_init (gpointer g_class);
@@ -151,7 +151,7 @@ gst_speexenc_init (GstSpeexEnc * speexenc)
   gst_element_add_pad (GST_ELEMENT (speexenc), speexenc->srcpad);
 
   speex_bits_init (&speexenc->bits);
-  speexenc->mode = &speex_nb_mode;
+  speexenc->mode = (SpeexMode *) & speex_nb_mode;
   speexenc->bufsize = 0;
   speexenc->packet_count = 0;
   speexenc->n_packets = 20;
