@@ -26,6 +26,8 @@
 #include <string.h>
 #include <gst/tag/tag.h>
 
+GST_DEBUG_CATEGORY (theoraenc_debug);
+#define GST_CAT_DEFAULT theoraenc_debug
 
 #define GST_TYPE_THEORA_ENC \
   (gst_theora_enc_get_type())
@@ -41,6 +43,33 @@
 typedef struct _GstTheoraEnc GstTheoraEnc;
 typedef struct _GstTheoraEncClass GstTheoraEncClass;
 
+typedef enum
+{
+  BORDER_NONE,
+  BORDER_BLACK,
+  BORDER_MIRROR
+}
+GstTheoraEncBorderMode;
+
+#define GST_TYPE_BORDER_MODE (gst_border_mode_get_type())
+static GType
+gst_border_mode_get_type (void)
+{
+  static GType border_mode_type = 0;
+  static GEnumValue border_mode[] = {
+    {BORDER_NONE, "BORDER_NONE", "No Border"},
+    {BORDER_BLACK, "BORDER_BLACK", "Black Border"},
+    {BORDER_MIRROR, "BORDER_MIRROR", "Mirror image in borders"},
+    {0, NULL, NULL},
+  };
+
+  if (!border_mode_type) {
+    border_mode_type =
+        g_enum_register_static ("GstTheoraEncBorderMode", border_mode);
+  }
+  return border_mode_type;
+}
+
 struct _GstTheoraEnc
 {
   GstElement element;
@@ -54,6 +83,9 @@ struct _GstTheoraEnc
   theora_info info;
   theora_comment comment;
 
+  gboolean center;
+  GstTheoraEncBorderMode border;
+
   gint video_bitrate;           /* bitrate target for Theora video */
   gint video_quality;           /* Theora quality selector 0 = low, 63 = high */
   gboolean quick;
@@ -64,11 +96,14 @@ struct _GstTheoraEnc
   gint keyframe_mindistance;
   gint noise_sensitivity;
 
+  gint info_width, info_height;
   gint width, height;
+  gint offset_x, offset_y;
   gdouble fps;
 
   guint packetno;
   guint64 bytes_out;
+  guint64 next_ts;
 };
 
 struct _GstTheoraEncClass
@@ -76,6 +111,12 @@ struct _GstTheoraEncClass
   GstElementClass parent_class;
 };
 
+#define ROUND_UP_2(x) (((x) + 1) & ~1)
+#define ROUND_UP_4(x) (((x) + 3) & ~3)
+#define ROUND_UP_8(x) (((x) + 7) & ~7)
+
+#define THEORA_DEF_CENTER 		TRUE
+#define THEORA_DEF_BORDER 		BORDER_BLACK
 #define THEORA_DEF_BITRATE 		0
 #define THEORA_DEF_QUALITY 		16
 #define THEORA_DEF_QUICK		TRUE
@@ -89,6 +130,8 @@ struct _GstTheoraEncClass
 enum
 {
   ARG_0,
+  ARG_CENTER,
+  ARG_BORDER,
   ARG_BITRATE,
   ARG_QUALITY,
   ARG_QUICK,
@@ -157,6 +200,15 @@ gst_theora_enc_class_init (GstTheoraEncClass * klass)
   gobject_class->set_property = theora_enc_set_property;
   gobject_class->get_property = theora_enc_get_property;
 
+  g_object_class_install_property (gobject_class, ARG_CENTER,
+      g_param_spec_boolean ("center", "Center",
+          "Center image when sizes not multiple of 16", THEORA_DEF_CENTER,
+          (GParamFlags) G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, ARG_BORDER,
+      g_param_spec_enum ("border", "Border",
+          "Border color to add when sizes not multiple of 16",
+          GST_TYPE_BORDER_MODE, THEORA_DEF_BORDER,
+          (GParamFlags) G_PARAM_READWRITE));
   /* general encoding stream options */
   g_object_class_install_property (gobject_class, ARG_BITRATE,
       g_param_spec_int ("bitrate", "Bitrate", "Compressed video bitrate (kbps)",
@@ -164,7 +216,6 @@ gst_theora_enc_class_init (GstTheoraEncClass * klass)
   g_object_class_install_property (gobject_class, ARG_QUALITY,
       g_param_spec_int ("quality", "Quality", "Video quality",
           0, 63, THEORA_DEF_QUALITY, (GParamFlags) G_PARAM_READWRITE));
-
   g_object_class_install_property (gobject_class, ARG_QUICK,
       g_param_spec_boolean ("quick", "Quick", "Quick encoding",
           THEORA_DEF_QUICK, (GParamFlags) G_PARAM_READWRITE));
@@ -194,6 +245,7 @@ gst_theora_enc_class_init (GstTheoraEncClass * klass)
           (GParamFlags) G_PARAM_READWRITE));
 
   gstelement_class->change_state = theora_enc_change_state;
+  GST_DEBUG_CATEGORY_INIT (theoraenc_debug, "theoraenc", 0, "Theora encoder");
 }
 
 static void
@@ -212,6 +264,9 @@ gst_theora_enc_init (GstTheoraEnc * enc)
   gst_pad_use_explicit_caps (enc->srcpad);
   gst_element_add_pad (GST_ELEMENT (enc), enc->srcpad);
 
+  enc->center = THEORA_DEF_CENTER;
+  enc->border = THEORA_DEF_BORDER;
+
   enc->video_bitrate = THEORA_DEF_BITRATE;
   enc->video_quality = THEORA_DEF_QUALITY;
   enc->quick = THEORA_DEF_QUICK;
@@ -230,6 +285,9 @@ theora_enc_sink_link (GstPad * pad, const GstCaps * caps)
 {
   GstStructure *structure = gst_caps_get_structure (caps, 0);
   GstTheoraEnc *enc = GST_THEORA_ENC (gst_pad_get_parent (pad));
+  const GValue *par;
+  GValue fps = { 0 };
+  GValue framerate = { 0 };
 
   if (!gst_caps_is_fixed (caps))
     return GST_PAD_LINK_DELAYED;
@@ -237,25 +295,46 @@ theora_enc_sink_link (GstPad * pad, const GstCaps * caps)
   gst_structure_get_int (structure, "width", &enc->width);
   gst_structure_get_int (structure, "height", &enc->height);
   gst_structure_get_double (structure, "framerate", &enc->fps);
-
-  /* Theora has a divisible-by-sixteen restriction for the encoded video size */
-  if ((enc->width & 0x0f) != 0 || (enc->height & 0x0f) != 0)
-    return GST_PAD_LINK_REFUSED;
+  par = gst_structure_get_value (structure, "pixel-aspect-ratio");
 
   theora_info_init (&enc->info);
-  enc->info.width = enc->width;
-  enc->info.height = enc->height;
+  /* Theora has a divisible-by-sixteen restriction for the encoded video size but
+   * we can define a visible area using the frame_width/frame_height */
+  enc->info_width = enc->info.width = (enc->width + 15) & ~15;
+  enc->info_height = enc->info.height = (enc->height + 15) & ~15;
   enc->info.frame_width = enc->width;
   enc->info.frame_height = enc->height;
-  enc->info.offset_x = 0;
-  enc->info.offset_y = 0;
 
-  /* do some scaling, we really need fractions in structures... */
-  enc->info.fps_numerator = enc->fps * 10000000;
-  enc->info.fps_denominator = 10000000;
-  enc->info.aspect_numerator = 1;
-  enc->info.aspect_denominator = 1;
-  /* */
+  /* center image if needed */
+  if (enc->center) {
+    /* make sure offset is even, for easier decoding */
+    enc->offset_x = ROUND_UP_2 (enc->info_width - enc->width) / 2;
+    enc->offset_y = ROUND_UP_2 (enc->info_height - enc->height) / 2;
+  } else {
+    enc->offset_x = 0;
+    enc->offset_y = 0;
+  }
+  enc->info.offset_x = enc->offset_x;
+  enc->info.offset_y = enc->offset_y;
+
+  /* convert double to fraction for the framerate */
+  g_value_init (&fps, G_TYPE_DOUBLE);
+  g_value_init (&framerate, GST_TYPE_FRACTION);
+  g_value_set_double (&fps, enc->fps);
+  g_value_transform (&fps, &framerate);
+
+  enc->info.fps_numerator = gst_value_get_fraction_numerator (&framerate);
+  enc->info.fps_denominator = gst_value_get_fraction_denominator (&framerate);
+
+  if (par) {
+    enc->info.aspect_numerator = gst_value_get_fraction_numerator (par);
+    enc->info.aspect_denominator = gst_value_get_fraction_denominator (par);
+  } else {
+    /* setting them to 0 indicates that the decoder can chose a good aspect
+     * ratio, defaulting to 1/1 */
+    enc->info.aspect_numerator = 0;
+    enc->info.aspect_denominator = 0;
+  }
 
   enc->info.colorspace = OC_CS_UNSPECIFIED;
   enc->info.target_bitrate = enc->video_bitrate;
@@ -278,7 +357,8 @@ theora_enc_sink_link (GstPad * pad, const GstCaps * caps)
 
 /* prepare a buffer for transmission by passing data through libtheora */
 static GstBuffer *
-theora_buffer_from_packet (GstTheoraEnc * enc, ogg_packet * packet)
+theora_buffer_from_packet (GstTheoraEnc * enc, ogg_packet * packet,
+    GstClockTime timestamp, GstClockTime duration)
 {
   GstBuffer *buf;
 
@@ -287,9 +367,14 @@ theora_buffer_from_packet (GstTheoraEnc * enc, ogg_packet * packet)
   memcpy (GST_BUFFER_DATA (buf), packet->packet, packet->bytes);
   GST_BUFFER_OFFSET (buf) = enc->bytes_out;
   GST_BUFFER_OFFSET_END (buf) = packet->granulepos;
-  GST_BUFFER_TIMESTAMP (buf) =
-      theora_granule_time (&enc->state, packet->granulepos) * GST_SECOND;
-  GST_BUFFER_DURATION (buf) = GST_SECOND / enc->fps;
+  GST_BUFFER_TIMESTAMP (buf) = timestamp;
+  GST_BUFFER_DURATION (buf) = duration;
+
+  /* the second most significant bit of the first data byte is cleared
+   * for keyframes */
+  if ((packet->packet[0] & 40) == 0) {
+    GST_BUFFER_FLAG_SET (buf, GST_BUFFER_KEY_UNIT);
+  }
 
   enc->packetno++;
 
@@ -310,11 +395,12 @@ theora_push_buffer (GstTheoraEnc * enc, GstBuffer * buffer)
 }
 
 static void
-theora_push_packet (GstTheoraEnc * enc, ogg_packet * packet)
+theora_push_packet (GstTheoraEnc * enc, ogg_packet * packet,
+    GstClockTime timestamp, GstClockTime duration)
 {
   GstBuffer *buf;
 
-  buf = theora_buffer_from_packet (enc, packet);
+  buf = theora_buffer_from_packet (enc, packet, timestamp, duration);
   theora_push_buffer (enc, buf);
 }
 
@@ -354,20 +440,28 @@ theora_enc_chain (GstPad * pad, GstData * data)
 {
   GstTheoraEnc *enc;
   ogg_packet op;
+  GstBuffer *buf;
+  GstClockTime in_time;
 
   enc = GST_THEORA_ENC (gst_pad_get_parent (pad));
   if (GST_IS_EVENT (data)) {
     switch (GST_EVENT_TYPE (data)) {
       case GST_EVENT_EOS:
         /* push last packet with eos flag */
-        if (theora_encode_packetout (&enc->state, 1, &op))
-          theora_push_packet (enc, &op);
+        while (theora_encode_packetout (&enc->state, 1, &op)) {
+          GstClockTime out_time =
+              theora_granule_time (&enc->state, op.granulepos) * GST_SECOND;
+          theora_push_packet (enc, &op, out_time, GST_SECOND / enc->fps);
+        }
       default:
         gst_pad_event_default (pad, GST_EVENT (data));
         return;
     }
   }
 
+  buf = GST_BUFFER (data);
+  in_time = GST_BUFFER_TIMESTAMP (buf);
+
   /* no packets written yet, setup headers */
   /* Theora streams begin with three headers; the initial header (with
      most of the codec setup parameters) which is mandated by the Ogg
@@ -380,17 +474,16 @@ theora_enc_chain (GstPad * pad, GstData * data)
     GstCaps *caps;
     GstBuffer *buf1, *buf2, *buf3;
 
-
     /* first packet will get its own page automatically */
     theora_encode_header (&enc->state, &op);
-    buf1 = theora_buffer_from_packet (enc, &op);
+    buf1 = theora_buffer_from_packet (enc, &op, 0, 0);
 
     /* create the remaining theora headers */
     theora_comment_init (&enc->comment);
     theora_encode_comment (&enc->comment, &op);
-    buf2 = theora_buffer_from_packet (enc, &op);
+    buf2 = theora_buffer_from_packet (enc, &op, 0, 0);
     theora_encode_tables (&enc->state, &op);
-    buf3 = theora_buffer_from_packet (enc, &op);
+    buf3 = theora_buffer_from_packet (enc, &op, 0, 0);
 
     /* mark buffers and put on caps */
     caps = gst_pad_get_caps (enc->srcpad);
@@ -398,11 +491,7 @@ theora_enc_chain (GstPad * pad, GstData * data)
 
     /* negotiate with these caps */
     GST_DEBUG ("here are the caps: %" GST_PTR_FORMAT, caps);
-    if (!gst_pad_set_explicit_caps (enc->srcpad, caps)) {
-      gst_caps_free (caps);
-      gst_data_unref (data);
-      return;
-    }
+    gst_pad_try_set_caps (enc->srcpad, caps);
 
     /* push out the header buffers */
     theora_push_buffer (enc, buf1);
@@ -412,30 +501,155 @@ theora_enc_chain (GstPad * pad, GstData * data)
 
   {
     yuv_buffer yuv;
+    gint res;
     gint y_size;
-    guchar *pixels;
+    guint8 *pixels;
 
-    pixels = GST_BUFFER_DATA (GST_BUFFER (data));
+    yuv.y_width = enc->info_width;
+    yuv.y_height = enc->info_height;
+    yuv.y_stride = enc->info_width;
 
-    yuv.y_width = enc->width;
-    yuv.y_height = enc->height;
-    yuv.y_stride = enc->width;
-
-    yuv.uv_width = enc->width / 2;
-    yuv.uv_height = enc->height / 2;
+    yuv.uv_width = enc->info_width / 2;
+    yuv.uv_height = enc->info_height / 2;
     yuv.uv_stride = yuv.uv_width;
 
-    y_size = enc->width * enc->height;
+    y_size = enc->info_width * enc->info_height;
 
-    yuv.y = pixels;
-    yuv.u = pixels + y_size;
-    yuv.v = pixels + y_size * 5 / 4;
+    if (enc->width == enc->info_width && enc->height == enc->info_height) {
+      /* easy case, no cropping/conversion needed */
+      pixels = GST_BUFFER_DATA (buf);
 
-    theora_encode_YUVin (&enc->state, &yuv);
-    if (theora_encode_packetout (&enc->state, 0, &op))
-      theora_push_packet (enc, &op);
+      yuv.y = pixels;
+      yuv.u = yuv.y + y_size;
+      yuv.v = yuv.u + y_size / 4;
+    } else {
+      GstBuffer *newbuf;
+      gint i;
+      guint8 *dest_y, *src_y;
+      guint8 *dest_u, *src_u;
+      guint8 *dest_v, *src_v;
+      gint src_y_stride, src_uv_stride;
+      gint dst_y_stride, dst_uv_stride;
+      gint width, height;
+      gint cwidth, cheight;
+      gint offset_x, right_x, right_border;
 
-    gst_data_unref (data);
+      /* source width/height */
+      width = enc->width;
+      height = enc->height;
+      /* soucre chroma width/height */
+      cwidth = width / 2;
+      cheight = height / 2;
+
+      /* source strides as defined in videotestsrc */
+      src_y_stride = ROUND_UP_4 (width);
+      src_uv_stride = ROUND_UP_8 (width) / 2;
+
+      /* destination strides from the real picture width */
+      dst_y_stride = enc->info_width;
+      dst_uv_stride = enc->info_width / 2;
+
+      newbuf = gst_pad_alloc_buffer (enc->srcpad,
+          GST_BUFFER_OFFSET_NONE, y_size * 3 / 2);
+
+      dest_y = yuv.y = GST_BUFFER_DATA (newbuf);
+      dest_u = yuv.u = yuv.y + y_size;
+      dest_v = yuv.v = yuv.u + y_size / 4;
+
+      src_y = GST_BUFFER_DATA (buf);
+      src_u = src_y + src_y_stride * ROUND_UP_2 (height);
+      src_v = src_u + src_uv_stride * ROUND_UP_2 (height) / 2;
+
+      if (enc->border != BORDER_NONE) {
+        /* fill top border */
+        for (i = 0; i < enc->offset_y; i++) {
+          memset (dest_y, 0, dst_y_stride);
+          dest_y += dst_y_stride;
+        }
+      } else {
+        dest_y += dst_y_stride * enc->offset_y;
+      }
+
+      offset_x = enc->offset_x;
+      right_x = width + enc->offset_x;
+      right_border = dst_y_stride - right_x;
+
+      /* copy Y plane */
+      for (i = 0; i < height; i++) {
+        memcpy (dest_y + offset_x, src_y, width);
+        if (enc->border != BORDER_NONE) {
+          memset (dest_y, 0, offset_x);
+          memset (dest_y + right_x, 0, right_border);
+        }
+
+        dest_y += dst_y_stride;
+        src_y += src_y_stride;
+      }
+
+      if (enc->border != BORDER_NONE) {
+        /* fill bottom border */
+        for (i = height + enc->offset_y; i < enc->info.height; i++) {
+          memset (dest_y, 0, dst_y_stride);
+          dest_y += dst_y_stride;
+        }
+
+        /* fill top border chroma */
+        for (i = 0; i < enc->offset_y / 2; i++) {
+          memset (dest_u, 128, dst_uv_stride);
+          memset (dest_v, 128, dst_uv_stride);
+          dest_u += dst_uv_stride;
+          dest_v += dst_uv_stride;
+        }
+      } else {
+        dest_u += dst_uv_stride * enc->offset_y / 2;
+        dest_v += dst_uv_stride * enc->offset_y / 2;
+      }
+
+      offset_x = enc->offset_x / 2;
+      right_x = cwidth + offset_x;
+      right_border = dst_uv_stride - right_x;
+
+      /* copy UV planes */
+      for (i = 0; i < cheight; i++) {
+        memcpy (dest_v + offset_x, src_v, cwidth);
+        memcpy (dest_u + offset_x, src_u, cwidth);
+
+        if (enc->border != BORDER_NONE) {
+          memset (dest_u, 128, offset_x);
+          memset (dest_u + right_x, 128, right_border);
+          memset (dest_v, 128, offset_x);
+          memset (dest_v + right_x, 128, right_border);
+        }
+
+        dest_u += dst_uv_stride;
+        dest_v += dst_uv_stride;
+        src_u += src_uv_stride;
+        src_v += src_uv_stride;
+      }
+
+      if (enc->border != BORDER_NONE) {
+        /* fill bottom border */
+        for (i = cheight + enc->offset_y / 2; i < enc->info_height / 2; i++) {
+          memset (dest_u, 128, dst_uv_stride);
+          memset (dest_v, 128, dst_uv_stride);
+          dest_u += dst_uv_stride;
+          dest_v += dst_uv_stride;
+        }
+      }
+
+      gst_buffer_unref (buf);
+      buf = newbuf;
+    }
+
+    res = theora_encode_YUVin (&enc->state, &yuv);
+    while (theora_encode_packetout (&enc->state, 0, &op)) {
+      GstClockTime out_time;
+
+      out_time = theora_granule_time (&enc->state, op.granulepos) * GST_SECOND;
+      theora_push_packet (enc, &op, out_time, GST_SECOND / enc->fps);
+    }
+
+    gst_buffer_unref (buf);
   }
 }
 
@@ -464,7 +678,6 @@ theora_enc_change_state (GstElement * element)
     case GST_STATE_READY_TO_NULL:
       break;
     default:
-      g_assert_not_reached ();
       break;
   }
 
@@ -478,6 +691,12 @@ theora_enc_set_property (GObject * object, guint prop_id,
   GstTheoraEnc *enc = GST_THEORA_ENC (object);
 
   switch (prop_id) {
+    case ARG_CENTER:
+      enc->center = g_value_get_boolean (value);
+      break;
+    case ARG_BORDER:
+      enc->border = g_value_get_enum (value);
+      break;
     case ARG_BITRATE:
       enc->video_bitrate = g_value_get_int (value) * 1000;
       enc->video_quality = 0;
@@ -520,6 +739,12 @@ theora_enc_get_property (GObject * object, guint prop_id,
   GstTheoraEnc *enc = GST_THEORA_ENC (object);
 
   switch (prop_id) {
+    case ARG_CENTER:
+      g_value_set_boolean (value, enc->center);
+      break;
+    case ARG_BORDER:
+      g_value_set_enum (value, enc->border);
+      break;
     case ARG_BITRATE:
       g_value_set_int (value, enc->video_bitrate / 1000);
       break;
