@@ -37,7 +37,7 @@ GstElementDetails vorbisenc_details = {
   "Ogg Vorbis encoder",
   "Codec/Encoder/Audio",
   "Encodes audio in OGG Vorbis format",
-  "Monty <monty@xiph.org>, " "Wim Taymans <wim.taymans@chello.be>",
+  "Monty <monty@xiph.org>, " "Wim Taymans <wim@fluendo.com>",
 };
 
 /* VorbisEnc signals and args */
@@ -54,11 +54,23 @@ enum
   ARG_BITRATE,
   ARG_MIN_BITRATE,
   ARG_QUALITY,
-  ARG_SERIAL,
   ARG_MANAGED,
-  ARG_LAST_MESSAGE,
+  ARG_LAST_MESSAGE
 };
 
+/* FIXME:
+ * vorbis_granule_time was added between 1.0 and 1.0.1; it's too silly
+ * to require a new version for such a simple function, but once we move
+ * beyond 1.0 for other reasons we can remove this copy */
+
+static double
+vorbis_granule_time_copy (vorbis_dsp_state * v, ogg_int64_t granulepos)
+{
+  if (granulepos >= 0)
+    return ((double) granulepos / v->vi->rate);
+  return (-1);
+}
+
 static const GstFormat *
 gst_vorbisenc_get_formats (GstPad * pad)
 {
@@ -81,6 +93,7 @@ gst_vorbisenc_get_formats (GstPad * pad)
 #define BITRATE_DEFAULT 	-1
 #define MIN_BITRATE_DEFAULT 	-1
 #define QUALITY_DEFAULT 	0.3
+#define LOWEST_BITRATE 8000     /* lowest allowed for a 8 kHz stream */
 
 static void gst_vorbisenc_base_init (gpointer g_class);
 static void gst_vorbisenc_class_init (VorbisEncClass * klass);
@@ -135,20 +148,20 @@ vorbisenc_get_type (void)
 static GstCaps *
 vorbis_caps_factory (void)
 {
-  return gst_caps_new_simple ("application/ogg", NULL);
+  return gst_caps_new_simple ("audio/x-vorbis", NULL);
 }
 
 static GstCaps *
 raw_caps_factory (void)
 {
+  /* lowest sample rate is in vorbis/lib/modes/setup_8.h, 8000 Hz
+   * highest sample rate is in vorbis/lib/modes/setup_44.h, 50000 Hz */
   return
-      gst_caps_new_simple ("audio/x-raw-int",
+      gst_caps_new_simple ("audio/x-raw-float",
+      "rate", GST_TYPE_INT_RANGE, 8000, 50000,
+      "channels", GST_TYPE_INT_RANGE, 1, 2, NULL,
       "endianness", G_TYPE_INT, G_BYTE_ORDER,
-      "signed", G_TYPE_BOOLEAN, TRUE,
-      "width", G_TYPE_INT, 16,
-      "depth", G_TYPE_INT, 16,
-      "rate", GST_TYPE_INT_RANGE, 11025, 48000,
-      "channels", GST_TYPE_INT_RANGE, 1, 2, NULL);
+      "width", G_TYPE_INT, 32, "buffer-frames", G_TYPE_INT, 0);
 }
 
 static void
@@ -181,25 +194,21 @@ gst_vorbisenc_class_init (VorbisEncClass * klass)
   gstelement_class = (GstElementClass *) klass;
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MAX_BITRATE,
-      g_param_spec_int ("max_bitrate", "Max bitrate",
-          " Specify a minimum bitrate (in bps). Useful for encoding for a fixed-size channel",
+      g_param_spec_int ("max-bitrate", "Maximum Bitrate",
+          "Specify a maximum bitrate (in bps). Useful for encoding for a fixed-size channel",
           -1, G_MAXINT, MAX_BITRATE_DEFAULT, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BITRATE,
-      g_param_spec_int ("bitrate", "Bitrate", "Choose a bitrate to encode at. "
-          "Attempt to encode at a bitrate averaging this. Takes an argument in kbps.",
+      g_param_spec_int ("bitrate", "Target Bitrate",
+          "Specify a target average bitrate (in bps). ",
           -1, G_MAXINT, BITRATE_DEFAULT, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MIN_BITRATE,
-      g_param_spec_int ("min_bitrate", "Min bitrate",
-          "Specify a maximum bitrate in bps. Useful for streaming applications.",
+      g_param_spec_int ("min_bitrate", "Minimum Bitrate",
+          "Specify a minimum bitrate (in bps).",
           -1, G_MAXINT, MIN_BITRATE_DEFAULT, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_QUALITY,
       g_param_spec_float ("quality", "Quality",
           "Specify quality instead of specifying a particular bitrate.",
           0.0, 1.0, QUALITY_DEFAULT, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SERIAL,
-      g_param_spec_int ("serial", "Serial",
-          "Specify a serial number for the stream. (-1 is random)", -1,
-          G_MAXINT, -1, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MANAGED,
       g_param_spec_boolean ("managed", "Managed",
           "Enable bitrate management engine", FALSE, G_PARAM_READWRITE));
@@ -222,6 +231,7 @@ gst_vorbisenc_sinkconnect (GstPad * pad, const GstCaps * caps)
   GstStructure *structure;
 
   vorbisenc = GST_VORBISENC (gst_pad_get_parent (pad));
+  vorbisenc->setup = FALSE;
 
   structure = gst_caps_get_structure (caps, 0);
   gst_structure_get_int (structure, "channels", &vorbisenc->channels);
@@ -350,6 +360,7 @@ gst_vorbisenc_get_query_types (GstPad * pad)
     GST_QUERY_POSITION,
     0
   };
+
   return gst_vorbisenc_src_query_types;
 }
 
@@ -456,7 +467,6 @@ gst_vorbisenc_init (VorbisEnc * vorbisenc)
   vorbisenc->min_bitrate = MIN_BITRATE_DEFAULT;
   vorbisenc->quality = QUALITY_DEFAULT;
   vorbisenc->quality_set = FALSE;
-  vorbisenc->serial = -1;
   vorbisenc->last_message = NULL;
 
   vorbisenc->setup = FALSE;
@@ -616,7 +626,7 @@ update_start_message (VorbisEnc * vorbisenc)
 static gboolean
 gst_vorbisenc_setup (VorbisEnc * vorbisenc)
 {
-  gint serial;
+  vorbisenc->setup = FALSE;
 
   if (vorbisenc->bitrate < 0 && vorbisenc->min_bitrate < 0
       && vorbisenc->max_bitrate < 0) {
@@ -631,9 +641,10 @@ gst_vorbisenc_setup (VorbisEnc * vorbisenc)
 
   if (vorbisenc->quality_set) {
     if (vorbis_encode_setup_vbr (&vorbisenc->vi,
-            vorbisenc->channels, vorbisenc->frequency, vorbisenc->quality)) {
-      g_warning
-          ("vorbisenc: initialisation failed: invalid parameters for quality");
+            vorbisenc->channels, vorbisenc->frequency,
+            vorbisenc->quality) != 0) {
+      GST_ERROR_OBJECT (vorbisenc,
+          "vorbisenc: initialisation failed: invalid parameters for quality");
       vorbis_info_clear (&vorbisenc->vi);
       return FALSE;
     }
@@ -644,7 +655,8 @@ gst_vorbisenc_setup (VorbisEnc * vorbisenc)
 
       vorbis_encode_ctl (&vorbisenc->vi, OV_ECTL_RATEMANAGE_GET, &ai);
 
-      /* the bitrates are in kHz */
+      /* the bitrates used by libvorbisenc are in kbit/sec, ours in bit/sec
+       * also remember that in telecom kbit/sec is 1000 bit/sec */
       ai.bitrate_hard_min = vorbisenc->min_bitrate / 1000;
       ai.bitrate_hard_max = vorbisenc->max_bitrate / 1000;
       ai.management_active = 1;
@@ -652,14 +664,20 @@ gst_vorbisenc_setup (VorbisEnc * vorbisenc)
       vorbis_encode_ctl (&vorbisenc->vi, OV_ECTL_RATEMANAGE_SET, &ai);
     }
   } else {
+    long min_bitrate, max_bitrate;
+
+    min_bitrate = vorbisenc->min_bitrate > 0 ? vorbisenc->min_bitrate : -1;
+    max_bitrate = vorbisenc->max_bitrate > 0 ? vorbisenc->max_bitrate : -1;
+
     if (vorbis_encode_setup_managed (&vorbisenc->vi,
             vorbisenc->channels,
             vorbisenc->frequency,
-            vorbisenc->max_bitrate > 0 ? vorbisenc->max_bitrate : -1,
-            vorbisenc->bitrate,
-            vorbisenc->min_bitrate > 0 ? vorbisenc->min_bitrate : -1)) {
-      g_warning
-          ("vorbisenc: initialisation failed: invalid parameters for bitrate\n");
+            max_bitrate, vorbisenc->bitrate, min_bitrate) != 0) {
+      GST_ERROR_OBJECT (vorbisenc,
+          "vorbis_encode_setup_managed "
+          "(c %d, rate %d, max br %ld, br %ld, min br %ld) failed",
+          vorbisenc->channels, vorbisenc->frequency, max_bitrate,
+          vorbisenc->bitrate, min_bitrate);
       vorbis_info_clear (&vorbisenc->vi);
       return FALSE;
     }
@@ -677,45 +695,82 @@ gst_vorbisenc_setup (VorbisEnc * vorbisenc)
   vorbis_analysis_init (&vorbisenc->vd, &vorbisenc->vi);
   vorbis_block_init (&vorbisenc->vd, &vorbisenc->vb);
 
-  /* set up our packet->stream encoder */
-  /* pick a random serial number; that way we can more likely build
-     chained streams just by concatenation */
-  if (vorbisenc->serial < 0) {
-    srand (time (NULL));
-    serial = rand ();
-  } else {
-    serial = vorbisenc->serial;
-  }
-
-  ogg_stream_init (&vorbisenc->os, serial);
-
   vorbisenc->setup = TRUE;
 
   return TRUE;
 }
 
-static void
-gst_vorbisenc_write_page (VorbisEnc * vorbisenc, ogg_page * page)
+/* prepare a buffer for transmission by passing data through libvorbis */
+static GstBuffer *
+gst_vorbisenc_buffer_from_packet (VorbisEnc * vorbisenc, ogg_packet * packet)
 {
   GstBuffer *outbuf;
 
-  outbuf = gst_buffer_new_and_alloc (page->header_len + page->body_len);
+  outbuf = gst_buffer_new_and_alloc (packet->bytes);
+  memcpy (GST_BUFFER_DATA (outbuf), packet->packet, packet->bytes);
+  GST_BUFFER_OFFSET (outbuf) = vorbisenc->bytes_out;
+  GST_BUFFER_OFFSET_END (outbuf) = packet->granulepos;
+  GST_BUFFER_TIMESTAMP (outbuf) =
+      vorbis_granule_time_copy (&vorbisenc->vd,
+      packet->granulepos) * GST_SECOND;
 
-  memcpy (GST_BUFFER_DATA (outbuf), page->header, page->header_len);
-  memcpy (GST_BUFFER_DATA (outbuf) + page->header_len,
-      page->body, page->body_len);
+  GST_DEBUG ("encoded buffer of %d bytes", GST_BUFFER_SIZE (outbuf));
+  return outbuf;
+}
 
-  GST_DEBUG ("vorbisenc: encoded buffer of %d bytes", GST_BUFFER_SIZE (outbuf));
-
-  vorbisenc->bytes_out += GST_BUFFER_SIZE (outbuf);
+/* push out the buffer and do internal bookkeeping */
+static void
+gst_vorbisenc_push_buffer (VorbisEnc * vorbisenc, GstBuffer * buffer)
+{
+  vorbisenc->bytes_out += GST_BUFFER_SIZE (buffer);
 
   if (GST_PAD_IS_USABLE (vorbisenc->srcpad)) {
-    gst_pad_push (vorbisenc->srcpad, GST_DATA (outbuf));
+    gst_pad_push (vorbisenc->srcpad, GST_DATA (buffer));
   } else {
-    gst_buffer_unref (outbuf);
+    gst_buffer_unref (buffer);
   }
 }
 
+static void
+gst_vorbisenc_push_packet (VorbisEnc * vorbisenc, ogg_packet * packet)
+{
+  GstBuffer *outbuf;
+
+  outbuf = gst_vorbisenc_buffer_from_packet (vorbisenc, packet);
+  gst_vorbisenc_push_buffer (vorbisenc, outbuf);
+}
+
+static void
+gst_vorbisenc_set_header_on_caps (GstCaps * caps, GstBuffer * buf1,
+    GstBuffer * buf2, GstBuffer * buf3)
+{
+  GstStructure *structure = gst_caps_get_structure (caps, 0);
+  GValue list = { 0 };
+  GValue value = { 0 };
+
+  /* mark buffers */
+  GST_BUFFER_FLAG_SET (buf1, GST_BUFFER_IN_CAPS);
+  GST_BUFFER_FLAG_SET (buf2, GST_BUFFER_IN_CAPS);
+  GST_BUFFER_FLAG_SET (buf3, GST_BUFFER_IN_CAPS);
+
+  /* put buffers in a fixed list */
+  g_value_init (&list, GST_TYPE_FIXED_LIST);
+  g_value_init (&value, GST_TYPE_BUFFER);
+  g_value_set_boxed (&value, buf1);
+  gst_value_list_append_value (&list, &value);
+  g_value_unset (&value);
+  g_value_init (&value, GST_TYPE_BUFFER);
+  g_value_set_boxed (&value, buf2);
+  gst_value_list_append_value (&list, &value);
+  g_value_unset (&value);
+  g_value_init (&value, GST_TYPE_BUFFER);
+  g_value_set_boxed (&value, buf3);
+  gst_value_list_append_value (&list, &value);
+  gst_structure_set_value (structure, "streamheader", &list);
+  g_value_unset (&value);
+  g_value_unset (&list);
+}
+
 static void
 gst_vorbisenc_chain (GstPad * pad, GstData * _data)
 {
@@ -738,6 +793,7 @@ gst_vorbisenc_chain (GstPad * pad, GstData * _data)
            Tell the library we're at end of stream so that it can handle
            the last frame and mark end of stream in the output properly */
         vorbis_analysis_wrote (&vorbisenc->vd, 0);
+        vorbisenc->eos = TRUE;
         gst_event_unref (event);
         break;
       case GST_EVENT_TAG:
@@ -754,7 +810,7 @@ gst_vorbisenc_chain (GstPad * pad, GstData * _data)
         return;
     }
   } else {
-    gint16 *data;
+    gfloat *data;
     gulong size;
     gulong i, j;
     float **buffer;
@@ -767,7 +823,7 @@ gst_vorbisenc_chain (GstPad * pad, GstData * _data)
     }
 
     if (!vorbisenc->header_sent) {
-      gint result;
+      //gint result;
 
       /* Vorbis streams begin with three headers; the initial header (with
          most of the codec setup parameters) which is mandated by the Ogg
@@ -778,23 +834,37 @@ gst_vorbisenc_chain (GstPad * pad, GstData * _data)
       ogg_packet header;
       ogg_packet header_comm;
       ogg_packet header_code;
+      GstBuffer *buf1, *buf2, *buf3;
+      GstCaps *caps;
 
       gst_vorbisenc_set_metadata (vorbisenc);
       vorbis_analysis_headerout (&vorbisenc->vd, &vorbisenc->vc, &header,
           &header_comm, &header_code);
-      ogg_stream_packetin (&vorbisenc->os, &header);    /* automatically placed in its own page */
-      ogg_stream_packetin (&vorbisenc->os, &header_comm);
-      ogg_stream_packetin (&vorbisenc->os, &header_code);
 
-      while ((result = ogg_stream_flush (&vorbisenc->os, &vorbisenc->og))) {
-        gst_vorbisenc_write_page (vorbisenc, &vorbisenc->og);
-      }
+      /* create header buffers */
+      buf1 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header);
+      buf2 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header_comm);
+      buf3 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header_code);
+
+      /* mark and put on caps */
+      caps = gst_pad_get_caps (vorbisenc->srcpad);
+      gst_vorbisenc_set_header_on_caps (caps, buf1, buf2, buf3);
+
+      /* negotiate with these caps */
+      GST_DEBUG ("here are the caps: %" GST_PTR_FORMAT, caps);
+      gst_pad_try_set_caps (vorbisenc->srcpad, caps);
+
+      /* push out buffers */
+      gst_vorbisenc_push_buffer (vorbisenc, buf1);
+      gst_vorbisenc_push_buffer (vorbisenc, buf2);
+      gst_vorbisenc_push_buffer (vorbisenc, buf3);
+
       vorbisenc->header_sent = TRUE;
     }
 
     /* data to encode */
-    data = (gint16 *) GST_BUFFER_DATA (buf);
-    size = GST_BUFFER_SIZE (buf) / (vorbisenc->channels * 2);
+    data = (gfloat *) GST_BUFFER_DATA (buf);
+    size = GST_BUFFER_SIZE (buf) / (vorbisenc->channels * sizeof (float));
 
     /* expose the buffer to submit data */
     buffer = vorbis_analysis_buffer (&vorbisenc->vd, size);
@@ -802,7 +872,7 @@ gst_vorbisenc_chain (GstPad * pad, GstData * _data)
     /* uninterleave samples */
     for (i = 0; i < size; i++) {
       for (j = 0; j < vorbisenc->channels; j++) {
-        buffer[j][i] = data[i * vorbisenc->channels + j] / 32768.f;
+        buffer[j][i] = *data++;
       }
     }
 
@@ -814,41 +884,23 @@ gst_vorbisenc_chain (GstPad * pad, GstData * _data)
     gst_buffer_unref (buf);
   }
 
-  /* vorbis does some data preanalysis, then divvies up blocks for
+  /* vorbis does some data preanalysis, then divides up blocks for
      more involved (potentially parallel) processing.  Get a single
      block for encoding now */
   while (vorbis_analysis_blockout (&vorbisenc->vd, &vorbisenc->vb) == 1) {
+    ogg_packet op;
 
     /* analysis */
     vorbis_analysis (&vorbisenc->vb, NULL);
     vorbis_bitrate_addblock (&vorbisenc->vb);
 
-    while (vorbis_bitrate_flushpacket (&vorbisenc->vd, &vorbisenc->op)) {
-
-      /* weld the packet into the bitstream */
-      ogg_stream_packetin (&vorbisenc->os, &vorbisenc->op);
-
-      /* write out pages (if any) */
-      while (!vorbisenc->eos) {
-        int result = ogg_stream_pageout (&vorbisenc->os, &vorbisenc->og);
-
-        if (result == 0)
-          break;
-
-        gst_vorbisenc_write_page (vorbisenc, &vorbisenc->og);
-
-        /* this could be set above, but for illustrative purposes, I do
-           it here (to show that vorbis does know where the stream ends) */
-        if (ogg_page_eos (&vorbisenc->og)) {
-          vorbisenc->eos = 1;
-        }
-      }
+    while (vorbis_bitrate_flushpacket (&vorbisenc->vd, &op)) {
+      gst_vorbisenc_push_packet (vorbisenc, &op);
     }
   }
 
   if (vorbisenc->eos) {
     /* clean up and exit.  vorbis_info_clear() must be called last */
-    ogg_stream_clear (&vorbisenc->os);
     vorbis_block_clear (&vorbisenc->vb);
     vorbis_dsp_clear (&vorbisenc->vd);
     vorbis_info_clear (&vorbisenc->vi);
@@ -881,9 +933,6 @@ gst_vorbisenc_get_property (GObject * object, guint prop_id, GValue * value,
     case ARG_QUALITY:
       g_value_set_float (value, vorbisenc->quality);
       break;
-    case ARG_SERIAL:
-      g_value_set_int (value, vorbisenc->serial);
-      break;
     case ARG_MANAGED:
       g_value_set_boolean (value, vorbisenc->managed);
       break;
@@ -913,6 +962,11 @@ gst_vorbisenc_set_property (GObject * object, guint prop_id,
       gboolean old_value = vorbisenc->managed;
 
       vorbisenc->max_bitrate = g_value_get_int (value);
+      if (vorbisenc->max_bitrate >= 0
+          && vorbisenc->max_bitrate < LOWEST_BITRATE) {
+        g_warning ("Lowest allowed bitrate is %d", LOWEST_BITRATE);
+        vorbisenc->max_bitrate = LOWEST_BITRATE;
+      }
       if (vorbisenc->min_bitrate > 0 && vorbisenc->max_bitrate > 0)
         vorbisenc->managed = TRUE;
       else
@@ -924,12 +978,21 @@ gst_vorbisenc_set_property (GObject * object, guint prop_id,
     }
     case ARG_BITRATE:
       vorbisenc->bitrate = g_value_get_int (value);
+      if (vorbisenc->bitrate >= 0 && vorbisenc->bitrate < LOWEST_BITRATE) {
+        g_warning ("Lowest allowed bitrate is %d", LOWEST_BITRATE);
+        vorbisenc->bitrate = LOWEST_BITRATE;
+      }
       break;
     case ARG_MIN_BITRATE:
     {
       gboolean old_value = vorbisenc->managed;
 
       vorbisenc->min_bitrate = g_value_get_int (value);
+      if (vorbisenc->min_bitrate >= 0
+          && vorbisenc->min_bitrate < LOWEST_BITRATE) {
+        g_warning ("Lowest allowed bitrate is %d", LOWEST_BITRATE);
+        vorbisenc->min_bitrate = LOWEST_BITRATE;
+      }
       if (vorbisenc->min_bitrate > 0 && vorbisenc->max_bitrate > 0)
         vorbisenc->managed = TRUE;
       else
@@ -946,9 +1009,6 @@ gst_vorbisenc_set_property (GObject * object, guint prop_id,
       else
         vorbisenc->quality_set = FALSE;
       break;
-    case ARG_SERIAL:
-      vorbisenc->serial = g_value_get_int (value);
-      break;
     case ARG_MANAGED:
       vorbisenc->managed = g_value_get_boolean (value);
       break;
