@@ -191,6 +191,7 @@ gst_riff_create_video_caps_with_data (guint32 codec_fcc,
         *codec_name = g_strdup ("Microsoft MPEG-4 4.1");
       break;
 
+    case GST_MAKE_FOURCC ('m', 'p', '4', '2'):
     case GST_MAKE_FOURCC ('M', 'P', '4', '2'):
       caps = gst_caps_new_simple ("video/x-msmpeg",
           "msmpegversion", G_TYPE_INT, 42, NULL);
@@ -198,6 +199,7 @@ gst_riff_create_video_caps_with_data (guint32 codec_fcc,
         *codec_name = g_strdup ("Microsoft MPEG-4 4.2");
       break;
 
+    case GST_MAKE_FOURCC ('m', 'p', '4', '3'):
     case GST_MAKE_FOURCC ('M', 'P', '4', '3'):
       caps = gst_caps_new_simple ("video/x-msmpeg",
           "msmpegversion", G_TYPE_INT, 43, NULL);
@@ -330,11 +332,14 @@ gst_riff_create_video_caps (guint32 codec_fcc,
 }
 
 GstCaps *
-gst_riff_create_audio_caps (guint16 codec_id,
-    gst_riff_strh * strh, gst_riff_strf_auds * strf, char **codec_name)
+gst_riff_create_audio_caps_with_data (guint16 codec_id,
+    gst_riff_strh * strh, gst_riff_strf_auds * strf,
+    GstBuffer * strf_data, GstBuffer * strd_data, char **codec_name)
 {
-  gboolean block_align = FALSE;
+  gboolean block_align = FALSE, rate_chan = TRUE;
   GstCaps *caps = NULL;
+  gint rate_min = 8000, rate_max = 96000;
+  gint channels_max = 2;
 
   switch (codec_id) {
     case GST_RIFF_WAVE_FORMAT_MPEGL3:  /* mp3 */
@@ -353,9 +358,9 @@ gst_riff_create_audio_caps (guint16 codec_id,
 
     case GST_RIFF_WAVE_FORMAT_PCM:     /* PCM */
       if (strf != NULL) {
-        gint ba = GUINT16_FROM_LE (strf->blockalign);
-        gint ch = GUINT16_FROM_LE (strf->channels);
-        gint ws = GUINT16_FROM_LE (strf->size);
+        gint ba = strf->blockalign;
+        gint ch = strf->channels;
+        gint ws = strf->size;
 
         caps = gst_caps_new_simple ("audio/x-raw-int",
             "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
@@ -419,35 +424,75 @@ gst_riff_create_audio_caps (guint16 codec_id,
       break;
 
     case GST_RIFF_WAVE_FORMAT_A52:
+      channels_max = 6;
       caps = gst_caps_new_simple ("audio/x-ac3", NULL);
       if (codec_name)
         *codec_name = g_strdup ("AC3");
       break;
+    case GST_RIFF_WAVE_FORMAT_WMAV1:
+    case GST_RIFF_WAVE_FORMAT_WMAV2:
+    {
+      gint version = codec_id == GST_RIFF_WAVE_FORMAT_WMAV1 ? 1 : 2;
 
+      block_align = TRUE;
+
+      caps = gst_caps_new_simple ("audio/x-wma",
+          "wmaversion", G_TYPE_INT, version, NULL);
+
+      if (codec_name)
+        *codec_name = g_strdup_printf ("WMA Version %d", version);
+
+      if (strf != NULL) {
+        gst_caps_set_simple (caps,
+            "bitrate", G_TYPE_INT, strf->av_bps * 8, NULL);
+      } else {
+        gst_caps_set_simple (caps,
+            "bitrate", GST_TYPE_INT_RANGE, 0, G_MAXINT, NULL);
+      }
+      if (strf_data) {
+        gst_caps_set_simple (caps,
+            "codec_data", GST_TYPE_BUFFER, strf_data, NULL);
+      }
+      break;
+    }
     default:
       GST_WARNING ("Unknown audio tag 0x%04x", codec_id);
       return NULL;
   }
 
   if (strf != NULL) {
-    gst_caps_set_simple (caps,
-        "rate", G_TYPE_INT, strf->rate,
-        "channels", G_TYPE_INT, strf->channels, NULL);
-    if (block_align)
+    if (rate_chan) {
+      gst_caps_set_simple (caps,
+          "rate", G_TYPE_INT, strf->rate,
+          "channels", G_TYPE_INT, strf->channels, NULL);
+    }
+    if (block_align) {
       gst_caps_set_simple (caps,
           "block_align", G_TYPE_INT, strf->blockalign, NULL);
+    }
   } else {
-    gst_caps_set_simple (caps,
-        "rate", GST_TYPE_INT_RANGE, 8000, 96000,
-        "channels", GST_TYPE_INT_RANGE, 1, 2, NULL);
-    if (block_align)
+    if (rate_chan) {
+      gst_caps_set_simple (caps,
+          "rate", GST_TYPE_INT_RANGE, rate_min, rate_max,
+          "channels", GST_TYPE_INT_RANGE, 1, channels_max, NULL);
+    }
+    if (block_align) {
       gst_caps_set_simple (caps,
           "block_align", GST_TYPE_INT_RANGE, 1, 8192, NULL);
+    }
   }
 
   return caps;
 }
 
+GstCaps *
+gst_riff_create_audio_caps (guint16 codec_id,
+    gst_riff_strh * strh, gst_riff_strf_auds * strf, char **codec_name)
+{
+  return gst_riff_create_audio_caps_with_data (codec_id,
+      strh, strf, NULL, NULL, codec_name);
+}
+
 GstCaps *
 gst_riff_create_iavs_caps (guint32 codec_fcc,
     gst_riff_strh * strh, gst_riff_strf_iavs * strf, char **codec_name)
@@ -495,6 +540,7 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('M', 'P', 'E', 'G'),
     GST_MAKE_FOURCC ('H', '2', '6', '3'),
     GST_MAKE_FOURCC ('D', 'I', 'V', 'X'),
+    GST_MAKE_FOURCC ('D', 'X', '5', '0'),
     GST_MAKE_FOURCC ('X', 'V', 'I', 'D'),
     GST_MAKE_FOURCC ('3', 'I', 'V', '1'),
     GST_MAKE_FOURCC ('c', 'v', 'i', 'd'),
@@ -529,6 +575,8 @@ gst_riff_create_audio_template_caps (void)
     GST_RIFF_WAVE_FORMAT_MULAW,
     GST_RIFF_WAVE_FORMAT_ADPCM,
     GST_RIFF_WAVE_FORMAT_DVI_ADPCM,
+    GST_RIFF_WAVE_FORMAT_WMAV1,
+    GST_RIFF_WAVE_FORMAT_WMAV2,
     /* FILL ME */
     0
   };
