@@ -28,13 +28,13 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <gstauparse.h>
+#include "gstauparse.h"
 #include <gst/audio/audio.h>
 
 /* elementfactory information */
 static GstElementDetails gst_auparse_details =
 GST_ELEMENT_DETAILS (".au parser",
-    "Codec/Parser/Audio",
+    "Codec/Demuxer/Audio",
     "Parse an .au file into raw audio",
     "Erik Walthinsen <omega@cse.ogi.edu>");
 
@@ -48,10 +48,14 @@ GST_STATIC_PAD_TEMPLATE ("sink",
 static GstStaticPadTemplate gst_auparse_src_template =
     GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
-    GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_AUDIO_INT_PAD_TEMPLATE_CAPS "; "
-        "audio/x-alaw, "
-        "rate = (int) [ 8000, 48000 ], " "channels = (int) [ 1, 2 ]")
+    GST_PAD_SOMETIMES,          /* FIXME: spider */
+    GST_STATIC_CAPS (GST_AUDIO_INT_PAD_TEMPLATE_CAPS "; "       /* 24-bit PCM is barely supported by gstreamer actually */
+        GST_AUDIO_FLOAT_PAD_TEMPLATE_CAPS "; "  /* 64-bit float is barely supported by gstreamer actually */
+        "audio/x-alaw, " "rate = (int) [ 8000, 192000 ], "
+        "channels = (int) [ 1, 2 ]" "; " "audio/x-mulaw, "
+        "rate = (int) [ 8000, 192000 ], " "channels = (int) [ 1, 2 ]" "; "
+        /* Nothing to decode those ADPCM streams for now */
+        "audio/x-adpcm, " "layout = (string) { g721, g722, g723_3, g723_5 }")
     );
 
 /* AuParse signals and args */
@@ -63,8 +67,8 @@ enum
 
 enum
 {
-  ARG_0,
-  /* FILL ME */
+  ARG_0
+      /* FILL ME */
 };
 
 static void gst_auparse_base_init (gpointer g_class);
@@ -73,6 +77,8 @@ static void gst_auparse_init (GstAuParse * auparse);
 
 static void gst_auparse_chain (GstPad * pad, GstData * _data);
 
+static GstElementStateReturn gst_auparse_change_state (GstElement * element);
+
 static GstElementClass *parent_class = NULL;
 
 /*static guint gst_auparse_signals[LAST_SIGNAL] = { 0 }; */
@@ -94,6 +100,7 @@ gst_auparse_get_type (void)
       0,
       (GInstanceInitFunc) gst_auparse_init,
     };
+
     auparse_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstAuParse", &auparse_info,
         0);
@@ -122,6 +129,8 @@ gst_auparse_class_init (GstAuParseClass * klass)
   gstelement_class = (GstElementClass *) klass;
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+
+  gstelement_class->change_state = gst_auparse_change_state;
 }
 
 static void
@@ -133,11 +142,13 @@ gst_auparse_init (GstAuParse * auparse)
   gst_element_add_pad (GST_ELEMENT (auparse), auparse->sinkpad);
   gst_pad_set_chain_function (auparse->sinkpad, gst_auparse_chain);
 
-  auparse->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
+  auparse->srcpad = NULL;
+#if 0                           /* FIXME: spider */
+  gst_pad_new_from_template (gst_static_pad_template_get
       (&gst_auparse_src_template), "src");
   gst_element_add_pad (GST_ELEMENT (auparse), auparse->srcpad);
   gst_pad_use_explicit_caps (auparse->srcpad);
+#endif
 
   auparse->offset = 0;
   auparse->size = 0;
@@ -154,8 +165,10 @@ gst_auparse_chain (GstPad * pad, GstData * _data)
   gchar *data;
   glong size;
   GstCaps *tempcaps;
-  gint law, depth;
-  gboolean sign;
+  gint law = 0, depth = 0, ieee = 0;
+  gchar layout[7];
+
+  layout[0] = 0;
 
   g_return_if_fail (pad != NULL);
   g_return_if_fail (GST_IS_PAD (pad));
@@ -175,35 +188,36 @@ gst_auparse_chain (GstPad * pad, GstData * _data)
     guint32 *head = (guint32 *) data;
 
     /* normal format is big endian (au is a Sparc format) */
-    if (GUINT32_FROM_BE (*head) == 0x2e736e64) {
+    if (GST_READ_UINT32_BE (head) == 0x2e736e64) {      /* ".snd" */
       head++;
       auparse->le = 0;
-      auparse->offset = GUINT32_FROM_BE (*head);
+      auparse->offset = GST_READ_UINT32_BE (head);
       head++;
-      auparse->size = GUINT32_FROM_BE (*head);
+      /* Do not trust size, could be set to -1 : unknown */
+      auparse->size = GST_READ_UINT32_BE (head);
       head++;
-      auparse->encoding = GUINT32_FROM_BE (*head);
+      auparse->encoding = GST_READ_UINT32_BE (head);
       head++;
-      auparse->frequency = GUINT32_FROM_BE (*head);
+      auparse->frequency = GST_READ_UINT32_BE (head);
       head++;
-      auparse->channels = GUINT32_FROM_BE (*head);
+      auparse->channels = GST_READ_UINT32_BE (head);
       head++;
 
       /* and of course, someone had to invent a little endian
        * version.  Used by DEC systems. */
-    } else if (GUINT32_FROM_LE (*head) == 0x0064732E) {
+    } else if (GST_READ_UINT32_LE (head) == 0x0064732E) {       /* other source say it is "dns." */
+      head++;
       auparse->le = 1;
+      auparse->offset = GST_READ_UINT32_LE (head);
       head++;
-      auparse->le = 0;
-      auparse->offset = GUINT32_FROM_LE (*head);
+      /* Do not trust size, could be set to -1 : unknown */
+      auparse->size = GST_READ_UINT32_LE (head);
       head++;
-      auparse->size = GUINT32_FROM_LE (*head);
+      auparse->encoding = GST_READ_UINT32_LE (head);
       head++;
-      auparse->encoding = GUINT32_FROM_LE (*head);
+      auparse->frequency = GST_READ_UINT32_LE (head);
       head++;
-      auparse->frequency = GUINT32_FROM_LE (*head);
-      head++;
-      auparse->channels = GUINT32_FROM_LE (*head);
+      auparse->channels = GST_READ_UINT32_LE (head);
       head++;
 
     } else {
@@ -212,57 +226,132 @@ gst_auparse_chain (GstPad * pad, GstData * _data)
       return;
     }
 
-    g_print
-        ("offset %ld, size %ld, encoding %ld, frequency %ld, channels %ld\n",
-        auparse->offset, auparse->size, auparse->encoding, auparse->frequency,
-        auparse->channels);
     GST_DEBUG
         ("offset %ld, size %ld, encoding %ld, frequency %ld, channels %ld",
         auparse->offset, auparse->size, auparse->encoding, auparse->frequency,
         auparse->channels);
 
+/*
+Docs :
+	http://www.opengroup.org/public/pubs/external/auformat.html
+	http://astronomy.swin.edu.au/~pbourke/dataformats/au/
+	Solaris headers : /usr/include/audio/au.h
+	libsndfile : src/au.c
+Samples :
+	http://www.tsp.ece.mcgill.ca/MMSP/Documents/AudioFormats/AU/Samples.html
+*/
+
     switch (auparse->encoding) {
-      case 1:
+
+      case 1:                  /* 8-bit ISDN mu-law G.711 */
         law = 1;
         depth = 8;
-        sign = FALSE;
         break;
-      case 2:
-        law = 0;
+      case 27:                 /* 8-bit ISDN  A-law G.711 */
+        law = 2;
         depth = 8;
-        sign = FALSE;
         break;
-      case 3:
-        law = 0;
+
+      case 2:                  /*  8-bit linear PCM */
+        depth = 8;
+        break;
+      case 3:                  /* 16-bit linear PCM */
         depth = 16;
-        sign = TRUE;
         break;
+      case 4:                  /* 24-bit linear PCM */
+        depth = 24;
+        break;
+      case 5:                  /* 32-bit linear PCM */
+        depth = 32;
+        break;
+
+      case 6:                  /* 32-bit IEEE floating point */
+        ieee = 1;
+        depth = 32;
+        break;
+      case 7:                  /* 64-bit IEEE floating point */
+        ieee = 1;
+        depth = 64;
+        break;
+
+      case 23:                 /* 4-bit CCITT G.721   ADPCM 32kbps -> modplug/libsndfile (compressed 8-bit mu-law) */
+        strcpy (layout, "g721");
+        break;
+      case 24:                 /* 8-bit CCITT G.722   ADPCM        -> rtp */
+        strcpy (layout, "g722");
+        break;
+      case 25:                 /* 3-bit CCITT G.723.3 ADPCM 24kbps -> rtp/xine/modplug/libsndfile */
+        strcpy (layout, "g723_3");
+        break;
+      case 26:                 /* 5-bit CCITT G.723.5 ADPCM 40kbps -> rtp/xine/modplug/libsndfile */
+        strcpy (layout, "g723_5");
+        break;
+
+      case 8:                  /* Fragmented sample data */
+      case 9:                  /* AU_ENCODING_NESTED */
+
+      case 10:                 /* DSP program */
+      case 11:                 /* DSP  8-bit fixed point */
+      case 12:                 /* DSP 16-bit fixed point */
+      case 13:                 /* DSP 24-bit fixed point */
+      case 14:                 /* DSP 32-bit fixed point */
+
+      case 16:                 /* AU_ENCODING_DISPLAY : non-audio display data */
+      case 17:                 /* AU_ENCODING_MULAW_SQUELCH */
+
+      case 18:                 /* 16-bit linear with emphasis */
+      case 19:                 /* 16-bit linear compressed (NeXT) */
+      case 20:                 /* 16-bit linear with emphasis and compression */
+
+      case 21:                 /* Music kit DSP commands */
+      case 22:                 /* Music kit DSP commands samples */
+
       default:
-        g_warning ("help!, dont know how to deal with this format yet\n");
+        GST_ELEMENT_ERROR (auparse, STREAM, FORMAT, (NULL), (NULL));
+        gst_buffer_unref (buf);
         return;
     }
 
+    auparse->srcpad =
+        gst_pad_new_from_template (gst_static_pad_template_get
+        (&gst_auparse_src_template), "src");
+    gst_pad_use_explicit_caps (auparse->srcpad);
+
     if (law) {
-      tempcaps = gst_caps_new_simple ("audio/x-alaw",
-          "rate", G_TYPE_INT, auparse->frequency,
-          "channels", G_TYPE_INT, auparse->channels, NULL);
+      tempcaps =
+          gst_caps_new_simple ((law == 1) ? "audio/x-mulaw" : "audio/x-alaw",
+          "rate", G_TYPE_INT, auparse->frequency, "channels", G_TYPE_INT,
+          auparse->channels, NULL);
+    } else if (ieee) {
+      tempcaps = gst_caps_new_simple ("audio/x-raw-float",
+          "width", G_TYPE_INT, depth,
+          "endianness", G_TYPE_INT,
+          auparse->le ? G_LITTLE_ENDIAN : G_BIG_ENDIAN, NULL);
+    } else if (layout[0]) {
+      tempcaps = gst_caps_new_simple ("audio/x-adpcm",
+          "layout", G_TYPE_STRING, layout, NULL);
     } else {
       tempcaps = gst_caps_new_simple ("audio/x-raw-int",
-          "endianness", G_TYPE_INT, G_BIG_ENDIAN,
-          "rate", G_TYPE_INT, auparse->frequency,
-          "channels", G_TYPE_INT, auparse->channels,
-          "depth", G_TYPE_INT, depth,
-          "width", G_TYPE_INT, depth, "signed", G_TYPE_BOOLEAN, sign, NULL);
+          "endianness", G_TYPE_INT,
+          auparse->le ? G_LITTLE_ENDIAN : G_BIG_ENDIAN, "rate", G_TYPE_INT,
+          auparse->frequency, "channels", G_TYPE_INT, auparse->channels,
+          "depth", G_TYPE_INT, depth, "width", G_TYPE_INT, depth, "signed",
+          G_TYPE_BOOLEAN, TRUE, NULL);
     }
 
     if (!gst_pad_set_explicit_caps (auparse->srcpad, tempcaps)) {
       gst_buffer_unref (buf);
+      gst_object_unref (GST_OBJECT (auparse->srcpad));
+      auparse->srcpad = NULL;
       return;
     }
 
+    gst_element_add_pad (GST_ELEMENT (auparse), auparse->srcpad);
+
     newbuf = gst_buffer_new ();
     GST_BUFFER_DATA (newbuf) = (gpointer) malloc (size - (auparse->offset));
-    memcpy (GST_BUFFER_DATA (newbuf), data + 24, size - (auparse->offset));
+    memcpy (GST_BUFFER_DATA (newbuf), data + (auparse->offset),
+        size - (auparse->offset));
     GST_BUFFER_SIZE (newbuf) = size - (auparse->offset);
 
     gst_buffer_unref (buf);
@@ -274,6 +363,27 @@ gst_auparse_chain (GstPad * pad, GstData * _data)
   gst_pad_push (auparse->srcpad, GST_DATA (buf));
 }
 
+static GstElementStateReturn
+gst_auparse_change_state (GstElement * element)
+{
+  GstAuParse *auparse = GST_AUPARSE (element);
+
+  switch (GST_STATE_TRANSITION (element)) {
+    case GST_STATE_PAUSED_TO_READY:
+      if (auparse->srcpad) {
+        gst_element_remove_pad (element, auparse->srcpad);
+        auparse->srcpad = NULL;
+      }
+      break;
+    default:
+      break;
+  }
+
+  if (parent_class->change_state)
+    return parent_class->change_state (element);
+
+  return GST_STATE_SUCCESS;
+}
 
 static gboolean
 plugin_init (GstPlugin * plugin)
