@@ -33,6 +33,9 @@
 #include "matroska-demux.h"
 #include "matroska-ids.h"
 
+GST_DEBUG_CATEGORY (matroskademux_debug);
+#define GST_CAT_DEFAULT matroskademux_debug
+
 enum
 {
   /* FILL ME */
@@ -159,6 +162,9 @@ gst_matroska_demux_class_init (GstMatroskaDemuxClass * klass)
   gstelement_class->change_state = gst_matroska_demux_change_state;
   gstelement_class->send_event = gst_matroska_demux_send_event;
   gstelement_class->set_clock = gst_matroska_demux_set_clock;
+
+  GST_DEBUG_CATEGORY_INIT (matroskademux_debug, "matroskademux", 0,
+      "Matroska demuxer");
 }
 
 static void
@@ -453,6 +459,7 @@ gst_matroska_demux_add_stream (GstMatroskaDemux * demux)
                 break;
               }
               videocontext->display_width = num;
+              GST_DEBUG ("display_width %" G_GUINT64_FORMAT, num);
               break;
             }
 
@@ -465,6 +472,7 @@ gst_matroska_demux_add_stream (GstMatroskaDemux * demux)
                 break;
               }
               videocontext->display_height = num;
+              GST_DEBUG ("display_height %" G_GUINT64_FORMAT, num);
               break;
             }
 
@@ -477,6 +485,7 @@ gst_matroska_demux_add_stream (GstMatroskaDemux * demux)
                 break;
               }
               videocontext->pixel_width = num;
+              GST_DEBUG ("pixel_width %" G_GUINT64_FORMAT, num);
               break;
             }
 
@@ -489,6 +498,7 @@ gst_matroska_demux_add_stream (GstMatroskaDemux * demux)
                 break;
               }
               videocontext->pixel_height = num;
+              GST_DEBUG ("pixel_height %" G_GUINT64_FORMAT, num);
               break;
             }
 
@@ -1053,6 +1063,10 @@ gst_matroska_demux_handle_src_event (GstPad * pad, GstEvent * event)
     case GST_EVENT_SEEK:
       return gst_matroska_demux_send_event (GST_ELEMENT (demux), event);
 
+      /* events we don't need to handle */
+    case GST_EVENT_NAVIGATION:
+      break;
+
     default:
       GST_WARNING ("Unhandled event of type %d", GST_EVENT_TYPE (event));
       res = FALSE;
@@ -1917,6 +1931,20 @@ gst_matroska_demux_parse_blockgroup (GstMatroskaDemux * demux,
         GST_WARNING ("FIXME: implement support for BlockDuration");
         break;
       }
+      case GST_MATROSKA_ID_REFERENCEBLOCK:{
+        /* FIXME: this segfaults
+           gint64 num;
+           if (!gst_ebml_read_sint (ebml, &id, &num)) {
+           res = FALSE;
+           break;
+           }
+           GST_WARNING ("FIXME: implement support for ReferenceBlock");
+         */
+        if (!gst_ebml_read_skip (ebml))
+          res = FALSE;
+        break;
+      }
+
 
       default:
         GST_WARNING ("Unknown entry 0x%x in blockgroup data", id);
@@ -2434,6 +2462,11 @@ gst_matroska_demux_video_caps (GstMatroskaTrackVideoContext * videocontext,
     for (i = 0; i < gst_caps_get_size (caps); i++) {
       structure = gst_caps_get_structure (caps, i);
       if (videocontext != NULL) {
+        GST_DEBUG ("video size %dx%d, target display size %dx%d (any unit)",
+            videocontext->pixel_width,
+            videocontext->pixel_height,
+            videocontext->display_width, videocontext->display_height);
+        /* pixel width and height are the w and h of the video in pixels */
         if (videocontext->pixel_width > 0 && videocontext->pixel_height > 0) {
           gint w = videocontext->pixel_width;
           gint h = videocontext->pixel_height;
@@ -2445,18 +2478,19 @@ gst_matroska_demux_video_caps (GstMatroskaTrackVideoContext * videocontext,
               "width", GST_TYPE_INT_RANGE, 16, 4096,
               "height", GST_TYPE_INT_RANGE, 16, 4096, NULL);
         }
-#if 0
-        if (videocontext->display_width > 0 && videocontext->display_height > 0) {
-          gint w =
-              100 * videocontext->display_width / videocontext->pixel_width;
-          gint h =
-              100 * videocontext->display_height / videocontext->pixel_height;
 
-          gst_structure_set (structure,
-              "pixel_width", G_TYPE_INT, w,
-              "pixel_height", G_TYPE_INT, h, NULL);
+        if (videocontext->display_width > 0 && videocontext->display_height > 0) {
+          int n, d;
+
+          /* calculate the pixel aspect ratio using the display and pixel w/h */
+          n = videocontext->display_width * videocontext->pixel_height;
+          d = videocontext->display_height * videocontext->pixel_width;
+          GST_DEBUG ("setting PAR to %d/%d", n, d);
+          gst_structure_set (structure, "pixel-aspect-ratio", GST_TYPE_FRACTION,
+              videocontext->display_width * videocontext->pixel_height,
+              videocontext->display_height * videocontext->pixel_width, NULL);
         }
-#endif
+
         if (context->default_duration > 0) {
           gfloat framerate = 1. * GST_SECOND / context->default_duration;
 
@@ -2471,8 +2505,6 @@ gst_matroska_demux_video_caps (GstMatroskaTrackVideoContext * videocontext,
         gst_structure_set (structure,
             "width", GST_TYPE_INT_RANGE, 16, 4096,
             "height", GST_TYPE_INT_RANGE, 16, 4096,
-            "pixel_width", GST_TYPE_INT_RANGE, 0, 255,
-            "pixel_height", GST_TYPE_INT_RANGE, 0, 255,
             "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE, NULL);
       }
     }
