@@ -24,9 +24,9 @@
 #endif
 #include <gst/gsttaginterface.h>
 #include "gsttageditingprivate.h"
+#include <stdlib.h>
 #include <string.h>
 
-
 GST_DEBUG_CATEGORY_STATIC (gst_vorbis_tag_debug);
 #define GST_CAT_DEFAULT gst_vorbis_tag_debug
 
@@ -319,9 +319,19 @@ gst_vorbis_tag_add (GstTagList * list, const gchar * tag, const gchar * value)
         gst_tag_list_add (list, GST_TAG_MERGE_APPEND, gst_tag, tmp, NULL);
       }
       break;
-    case G_TYPE_STRING:
-      gst_tag_list_add (list, GST_TAG_MERGE_APPEND, gst_tag, value, NULL);
+    case G_TYPE_STRING:{
+      gchar *valid;
+
+      if (!g_utf8_validate (value, -1, (const gchar **) &valid)) {
+        valid = g_strndup (value, valid - value);
+        GST_DEBUG ("Invalid vorbis comment tag, truncated it to %s\n", valid);
+      } else {
+        valid = g_strdup (value);
+      }
+      gst_tag_list_add (list, GST_TAG_MERGE_APPEND, gst_tag, valid, NULL);
+      g_free (valid);
       break;
+    }
     case G_TYPE_DOUBLE:
       gst_tag_list_add (list, GST_TAG_MERGE_APPEND, gst_tag, g_strtod (value,
               NULL), NULL);
@@ -353,7 +363,7 @@ gst_tag_list_from_vorbiscomment_buffer (const GstBuffer * buffer,
   data += x;									\
   size -= x;									\
   if (size < 4) goto error;							\
-  cur_size = GUINT32_FROM_LE (*((guint32 *) data));				\
+  cur_size = GST_READ_UINT32_LE (data);						\
   data += 4;									\
   size -= 4;									\
   if (cur_size > size) goto error;						\
@@ -415,7 +425,8 @@ typedef struct
   guint count;
   guint data_count;
   GList *entries;
-} MyForEach;
+}
+MyForEach;
 
 GList *
 gst_tag_to_vorbis_comments (const GstTagList * list, const gchar * tag)
